@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Novell, Inc.
+// Copyright (C) 2009-2010 Novell, Inc.
 // Copyright (C) 2009 Robert Dyer
 //
 // This program is free software; you can redistribute it and/or
@@ -16,6 +16,7 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
 using System;
+using System.Linq;
 using System.Collections.Generic;
 using System.IO;
 
@@ -54,6 +55,7 @@ namespace PdfMod.Gui
         public Gtk.Window        Window        { get; private set; }
         public DocumentIconView  IconView      { get; private set; }
         public MetadataEditorBox EditorBox     { get; private set; }
+        public BookmarkView      BookmarkView  { get; private set; }
 
         static Client ()
         {
@@ -90,22 +92,28 @@ namespace PdfMod.Gui
 
             // PDF Icon View
             IconView = new DocumentIconView (this);
-            var iconview_sw = new Gtk.ScrolledWindow () { Child = IconView };
+            var iconview_sw = new Gtk.ScrolledWindow ();
+            iconview_sw.AddWithViewport (IconView);
 
             query_box = new QueryBox (this) { NoShowAll = true };
             query_box.Hide ();
 
+            // ActionManager
+            ActionManager = new Hyena.Gui.ActionManager ();
+            Window.AddAccelGroup (ActionManager.UIManager.AccelGroup);
+            Actions = new Actions (this, ActionManager);
+
             // Status bar
             StatusBar = new Gtk.Statusbar () { HasResizeGrip = true };
             status_label = new Label () { Xalign = 0.0f };
             StatusBar.PackStart (status_label, true, true, 6);
             StatusBar.ReorderChild (status_label, 0);
 
-            // ActionManager
-            ActionManager = new Hyena.Gui.ActionManager ();
-            Window.AddAccelGroup (ActionManager.UIManager.AccelGroup);
-            Actions = new Actions (this, ActionManager);
+            var zoom_slider = new ZoomSlider (this);
+            StatusBar.PackEnd (zoom_slider, false, false, 0);
+            StatusBar.ReorderChild (zoom_slider, 1);
 
+            // Properties editor box
             EditorBox = new MetadataEditorBox (this) { NoShowAll = true };
             EditorBox.Hide ();
 
@@ -120,12 +128,22 @@ namespace PdfMod.Gui
             HeaderToolbar.NoShowAll = true;
             HeaderToolbar.Visible = Configuration.ShowToolbar;
 
+            // BookmarksView
+            BookmarkView = new BookmarkView (this);
+            BookmarkView.NoShowAll = true;
+            BookmarkView.Visible = false;
+
             var vbox = new VBox ();
             vbox.PackStart (menu_bar, false, false, 0);
             vbox.PackStart (HeaderToolbar, false, false, 0);
             vbox.PackStart (EditorBox, false, false, 0);
             vbox.PackStart (query_box, false, false, 0);
-            vbox.PackStart (iconview_sw, true, true, 0);
+
+            var hbox = new HPaned ();
+            hbox.Add1 (BookmarkView);
+            hbox.Add2 (iconview_sw);
+            vbox.PackStart (hbox, true, true, 0);
+
             vbox.PackStart (StatusBar, false, true, 0);
             Window.Add (vbox);
 
@@ -183,7 +201,7 @@ namespace PdfMod.Gui
 
         bool PromptIfUnsavedChanges ()
         {
-            if (Document != null && Document.HasUnsavedChanged) {
+            if (Document != null && Document.HasUnsavedChanges) {
                 var dialog = new Hyena.Widgets.HigMessageDialog (
                     Window, DialogFlags.Modal, MessageType.Warning, ButtonsType.None,
                     Catalog.GetString ("Save the changes made to this document?"),
@@ -238,7 +256,7 @@ namespace PdfMod.Gui
             }
         }
 
-        public override void LoadPath (string path, string suggestedFilename)
+        public override void LoadPath (string path, string suggestedFilename, System.Action finishedCallback)
         {
             lock (this) {
                 // One document per window
@@ -249,11 +267,11 @@ namespace PdfMod.Gui
 
                 loading = true;
             }
-            
+
             if (!path.StartsWith ("file://")) {
                 path = System.IO.Path.GetFullPath (path);
             }
-            
+
             Configuration.LastOpenFolder = System.IO.Path.GetDirectoryName (suggestedFilename ?? path);
             status_label.Text = Catalog.GetString ("Loading document...");
 
@@ -265,16 +283,18 @@ namespace PdfMod.Gui
                         Document.SuggestedSavePath = suggestedFilename;
                     }
 
-                    ThreadAssist.ProxyToMain (delegate {
+                    ThreadAssist.BlockingProxyToMain (delegate {
                         IconView.SetDocument (Document);
+                        BookmarkView.SetDocument (Document);
                         RecentManager.Default.AddItem (Document.Uri);
+
                         Document.Changed += UpdateForDocument;
                         UpdateForDocument ();
                         OnDocumentLoaded ();
                     });
                 } catch (Exception e) {
                     Document = null;
-                    ThreadAssist.ProxyToMain (delegate {
+                    ThreadAssist.BlockingProxyToMain (delegate {
                         status_label.Text = "";
                         if (e is System.IO.FileNotFoundException) {
                             try {
@@ -292,12 +312,16 @@ namespace PdfMod.Gui
                     lock (this) {
                         loading = false;
                     }
+
+                    if (finishedCallback != null)
+                        finishedCallback ();
                 }
             });
         }
 
         void UpdateForDocument ()
         {
+            ThreadAssist.AssertInMainThread ();
             var current_size = Document.FileSize;
             string size_str = null;
             if (original_size_string == null) {
@@ -322,7 +346,7 @@ namespace PdfMod.Gui
 
             var title = Document.Title;
             var filename = Document.Filename;
-            if (Document.HasUnsavedChanged) {
+            if (Document.HasUnsavedChanges) {
                 filename = "*" + filename;
             }
             Window.Title = title == null ? filename : String.Format ("{0} - {1}", filename, title);
@@ -333,9 +357,7 @@ namespace PdfMod.Gui
             // This method is called from some random thread, but we need
             // to do the dialog on the GUI thread; use the reset_event
             // to block this thread until the user is done with the dialog.
-            var reset_event = new System.Threading.ManualResetEvent (false);
-
-            ThreadAssist.ProxyToMain (delegate {
+            ThreadAssist.BlockingProxyToMain (delegate {
                 Log.Debug ("Password requested to open document");
                 var dialog = new Hyena.Widgets.HigMessageDialog (
                     Window, DialogFlags.Modal, MessageType.Question, ButtonsType.None,
@@ -361,10 +383,33 @@ namespace PdfMod.Gui
                     Log.Information ("Password dialog cancelled");
                     args.Abort = true;
                 }
-                reset_event.Set ();
             });
+        }
 
-            reset_event.WaitOne ();
+        public Gtk.FileChooserDialog CreateChooser (string title, FileChooserAction action)
+        {
+            var chooser = new Gtk.FileChooserDialog (title, this.Window, action) {
+                DefaultResponse = ResponseType.Ok
+            };
+            chooser.AddButton (Stock.Cancel, ResponseType.Cancel);
+            chooser.AddFilter (GtkUtilities.GetFileFilter (Catalog.GetString ("PDF Documents"), new string [] {"pdf"}));
+            chooser.AddFilter (GtkUtilities.GetFileFilter (Catalog.GetString ("All Files"), new string [] {"*"}));
+
+            var dirs = new string [] { "DOWNLOAD", "DOCUMENTS" }.Select (s => GetXdgDir (s))
+                                                                .Where (d => d != null)
+                                                                .ToArray ();
+            Hyena.Gui.GtkUtilities.SetChooserShortcuts (chooser, dirs);
+
+            return chooser;
+        }
+
+        private string GetXdgDir (string type)
+        {
+            try {
+                return XdgBaseDirectorySpec.GetXdgDirectoryUnderHome (String.Format ("XDG_{0}_DIR", type), null);
+            } catch {
+                return null;
+            }
         }
 
         static void OnLogNotify (LogNotifyArgs args)
