@@ -1,4 +1,4 @@
-// Copyright (C) 2009 Novell, Inc.
+// Copyright (C) 2009-2010 Novell, Inc.
 // Copyright (C) 2009 Julien Rebetez
 // Copyright (C) 2009 Michael McKinley
 //
@@ -144,7 +144,8 @@ namespace PdfMod.Pdf
             Uri = uri_obj.AbsoluteUri;
             SuggestedSavePath = Path = uri_obj.LocalPath;
 
-            pdf_document = PdfSharp.Pdf.IO.PdfReader.Open (Path, PdfDocumentOpenMode.Modify | PdfDocumentOpenMode.Import, passwordProvider);
+            pdf_document = PdfSharp.Pdf.IO.PdfReader.Open (Path, PdfDocumentOpenMode.Modify, passwordProvider);
+
             for (int i = 0; i < pdf_document.PageCount; i++) {
                 var page = new Page (pdf_document.Pages[i]) {
                     Document = this,
@@ -158,8 +159,17 @@ namespace PdfMod.Pdf
             OnChanged ();
         }
 
-        public bool HasUnsavedChanged {
-            get { return tmp_uri != null || save_timeout_id != 0; }
+        public bool HasUnsavedChanges {
+            get { return UnsavedChanges > 0 || tmp_uri != null || save_timeout_id != 0; }
+        }
+
+        int unsaved_changes = 0;
+        public int UnsavedChanges {
+            get { return unsaved_changes; }
+            set {
+                unsaved_changes = value;
+                OnChanged ();
+            }
         }
 
         public long FileSize {
@@ -258,8 +268,12 @@ namespace PdfMod.Pdf
         public void Save (string uri)
         {
             Pdf.Save (uri);
+            UnsavedChanges = 0;
             Log.DebugFormat ("Saved to {0}", uri);
+
+            var uri_obj = new Uri (uri);
             Uri = uri;
+            SuggestedSavePath = Path = uri_obj.LocalPath;
 
             if (tmp_uri != null) {
                 try {
@@ -270,6 +284,13 @@ namespace PdfMod.Pdf
                     tmp_uri = tmp_path = null;
                 }
             }
+
+            OnChanged ();
+        }
+
+        public void AppendFromUri (Uri uri)
+        {
+            AddFromUri (uri, Count);
         }
 
         public void AddFromUri (Uri uri)
